#!/bin/bash

################################################################################
# PathByAppendingNumberIfNeeded "parent_dir" "file_name" "file_extension"
################################################################################

function PathByAppendingNumberIfNeeded
{	
	SRC_PARENT_DIR="$1"
	SRC_FILE_NAME="$2"
	SRC_EXTENSION="$3"
	
	TEST_FILE_NAME="$SRC_FILE_NAME"
	TEST_PATH=""
	for (( c=2 ; ; c++ )); do
	
        TEST_PATH="$SRC_PARENT_DIR/$TEST_FILE_NAME.$SRC_EXTENSION"
        
        if [ ! -e "$TEST_PATH" ]; then
            break
        fi
        
        TEST_FILE_NAME="$SRC_FILE_NAME $c"
	
	done
		
	echo "$TEST_PATH"
}

################################################################################
# AbortWithError "error"
################################################################################

function AbortWithError()
{
    echo "$1"
	
	rm -rf "$TEMP_DIR"
    
    # Go back to the old installation if we created one
    
    if [ -e "/Library/Nopali.old" ]; then
    
        # Delete the new installation if we created one
        
        if [ -e "/Library/Nopali" ]; then
            # Stop the new server, if it's running
            /Library/Nopali/utils/nopali_utility stop
    
            rm -rf /Library/Nopali
        fi
        
        mv /Library/Nopali.old /Library/Nopali
        
    fi

	# Start up the old server
	/Library/Nopali/utils/nopali_utility start
    
    exit 1
}

################################################################################
# InstallServerBinaries "nopali_utility_path" "archive_path" "temp_dir"
################################################################################

function InstallServerBinaries()
{
	local nopali_utility_path=$1
	local archive_path=$2
	local temp_dir=$3

	if [ ! -e "${nopali_utility_path}" ]; then
		return 1
	fi
	
	if [ ! -e "${archive_path}" ]; then
		return 1
	fi

	if [ ! -d "${temp_dir}" ]; then
		return 1
	fi
	
	# Get the current OID of the database
		
	local old_database_oid=$(/Library/Nopali/bin/psql -U nopali -d nopali -A -t -c "SELECT oid from pg_database where datname='nopali';")
	
	if [ "${old_database_oid}" = "" ]; then
		return 1
	fi
	
	# Dump the server to a temp backup (no blobs)
	
	local backup_path="${temp_dir}/BackupWithoutBlobs.app"

	"${nopali_utility_path}" backup --database-only --skip-blobs --path "${backup_path}" || return 1

	# Shut down the server

	"${nopali_utility_path}" stop || return 1
	
	# Move /Library/Nopali to /Library/Nopali.old
	
	mv /Library/Nopali /Library/Nopali.old || return 1
	
	local old_fslo_dir="/Library/Nopali.old/data/fslo/${old_database_oid}"
	
	if [ ! -d "${old_fslo_dir}" ]; then
		return 1
	fi
	
	# Install the 6.5 binaries
	
	tar -xjf "${archive_path}" -C /Library || return 1
	cp "${nopali_utility_path}" /Library/Nopali/utils || return 1
		
	chown -R root:admin /Library/nopali || return 1
	
	# Restore the temp backup (no blobs)
	
	"${nopali_utility_path}" restore --database-only --skip-blobs --preserve-uuid --path "${backup_path}" || return 1
	
	# Move the blobs from the old data directory to the new one
	
	local new_database_oid=$(/Library/Nopali/bin/psql -U nopali -d nopali -A -t -c "SELECT oid from pg_database where datname='nopali';")
	
	if [ "${new_database_oid}" = "" ]; then
		return 1
	fi
	
	local new_fslo_dir="/Library/Nopali/data/fslo/${new_database_oid}"
	
	rm -rf "${new_fslo_dir}" || return 1
	mv "${old_fslo_dir}" "${new_fslo_dir}" || return 1
	
	return 0
}

################################################################################
#  1: Preparing to install
################################################################################

TEMP_DIR=$(mktemp -d -q /tmp/NopaliUpdater.XXXXXX)

# Display some computer info so it will be present in the Run Log

system_profiler SPHardwareDataType SPSoftwareDataType

# Display total number of steps so the GUI can react when all steps have ended
# Display source server version
# Display target server version


RESOURCES_DIR="$1"

BUNDLE_NOPALI_UTILITY="$2"

REQUIRED_SERVER_VERSION="$3"

FUTURE_SERVER_VERSION="$4"

DEBUG_OPTION="$5"

TOTAL_STEPS="5"


# Log the input parameters
echo "NU:REQUIREDSERVERVERSION: $REQUIRED_SERVER_VERSION."
echo "NU:FUTURESERVERVERSION: $FUTURE_SERVER_VERSION."
echo "NU:TOTALSTEPS: $TOTAL_STEPS"


echo "NU:STEPSTARTING:1:PREPARING_INSTALLATION"

if [ "$(id -u)" != "0" ]; then
    AbortWithError "NU:FATALERROR:Could not run the updater script with root privileges."
fi

if [ "${RESOURCES_DIR}" = "" ]; then
    RESOURCES_DIR="."
fi

if [ "${DEBUG_OPTION}" != "" ]; then
	export NOPALI_DEBUG=1
fi

EXISTING_NOPALI_UTILITY="/Library/Nopali/utils/nopali_utility"

BUNDLE_NOPALI_SERVER_DIR=$(dirname "$BUNDLE_NOPALI_UTILITY")

BUNDLE_NOPALI_SERVER="${BUNDLE_NOPALI_SERVER_DIR}/nopali_server"

EXISTING_NOPALI_SERVER="/Library/Nopali/utils/nopali_server"

#
# Make sure the Resources directory exists
#

if [ ! -d "${RESOURCES_DIR}" ]; then
    AbortWithError "NU:FATALERROR:Could not find the Resources directory: $RESOURCES_DIR"
fi

#
# Make sure the FUTURE_SERVER_VERSION.txt file exists
#

if [ ! -f "${RESOURCES_DIR}/${FUTURE_SERVER_VERSION}.txt" ]; then
    AbortWithError "NU:FATALERROR:Could not find the $FUTURE_SERVER_VERSION.txt file."
fi

#
# Make sure the nopali_utility file exists
#

if [ ! -f "${BUNDLE_NOPALI_UTILITY}" ]; then
    AbortWithError "NU:FATALERROR:Could not find the bundle nopali_utility file."
fi

# Make sure the server is running and the server_version is the
#     correct version

#
# Start up the database server if it isn't running
#

echo "Starting database server"
"${EXISTING_NOPALI_UTILITY}" start

SERVER_VERSION=$("${EXISTING_NOPALI_UTILITY}" info --server-version)

if [ "$?" != "0" ]; then
    AbortWithError "NU:FATALERROR:Could not connect to the database server."
fi


if [ "${SERVER_VERSION}" != "${REQUIRED_SERVER_VERSION}" ]; then
    AbortWithError "NU:FATALERROR:The database server is not the required version for this update ($REQUIRED_SERVER_VERSION)."
fi

# Display the staff table and the practice_data table so it will be present
# in the Run Log

/Library/Nopali/bin/psql nopali nopali -c "SELECT row_id, label, is_doctor, is_active FROM staff ORDER BY is_active, is_doctor, label"

/Library/Nopali/bin/psql nopali nopali -c "SELECT * FROM practice_data"

# Done with Step 1
echo "NU:STEPCOMPLETED:1"


################################################################################
#  2: Backing up database contents
echo "NU:STEPSTARTING:2:DATABASE_BACKUP"
################################################################################

# Done with Step 2
echo "NU:STEPCOMPLETED:2"


################################################################################
#  3: Updating database contents
echo "NU:STEPSTARTING:3:UPDATE_DATABASE_CONTENTS"
################################################################################

if [ "${FUTURE_SERVER_VERSION}" = "6.5" ]; then

	InstallServerBinaries "${BUNDLE_NOPALI_UTILITY}" "${RESOURCES_DIR}/6.5-Library-Nopali.tar.bz2" "${TEMP_DIR}"
	
	if [ "$?" != "0" ]; then
		AbortWithError "NU:FATALERROR:Error installing files for version 6.5"
	fi

fi

# Run the schema changes

echo "Running schema changes"
/Library/Nopali/bin/psql -U nopali -d nopali -f "${RESOURCES_DIR}/${FUTURE_SERVER_VERSION}.txt"

if [ "$?" != "0" ]; then
    AbortWithError "NU:FATALERROR:Error updating database contents"
fi

# Done with Step 3
echo "NU:STEPCOMPLETED:3"


################################################################################
#  4: Installing new files
echo "NU:STEPSTARTING:4:INSTALLING_NEW_FILES"
################################################################################

# Install the nopali_utility binary

rm -f "${EXISTING_NOPALI_UTILITY}"

cp "${BUNDLE_NOPALI_UTILITY}" /Library/Nopali/utils

if [ "$?" != "0" ]; then
	AbortWithError "NU:FATALERROR:Error installing nopali_utility"
fi

chown root:admin "${EXISTING_NOPALI_UTILITY}"

# Install the nopali_server binary

rm -f "${EXISTING_NOPALI_SERVER}"

cp "${BUNDLE_NOPALI_SERVER}" /Library/Nopali/utils

if [ "$?" != "0" ]; then
    AbortWithError "NU:FATALERROR:Error installing nopali_server"
fi

chown root:admin "${EXISTING_NOPALI_SERVER}"

# Restart the server (including nopali_server)

/Library/Nopali/utils/nopali_utility stop

if [ "$?" != "0" ]; then
    AbortWithError "NU:FATALERROR:Error stopping the server"
fi

/Library/Nopali/utils/nopali_utility start

if [ "$?" != "0" ]; then
    AbortWithError "NU:FATALERROR:Error starting the server"
fi

# Done with Step 4
echo "NU:STEPCOMPLETED:4"


################################################################################
#  5: Finishing installation
echo "NU:STEPSTARTING:5:FINISHING_INSTALLATION"
################################################################################

# Get rid of the temp directory

rm -rf "$TEMP_DIR"

# Delete /Library/Nopali.old

if [ -e "/Library/Nopali.old" ]; then
	rm -rf /Library/Nopali.old
fi

# Done with Step 5
echo "NU:STEPCOMPLETED:5"

# Done with Script
echo "NU:SCRIPTCOMPLETED"

exit 0
